require 'cuvva_error'
require 'json'

class Log
	ERROR_LEVELS = {
		debug: 0,
		info: 1,
		warn: 2,
		error: 3,
		fatal: 4,
	}

	@@conf_min_log_level = :info
	@@conf_handlers = {}

	def self.set_handler(*args)
		if args.length == 1
			handler = args[0]

			for level in ERROR_LEVELS.keys
				@@conf_handlers[level] = handler
			end
		else
			level, handler = args
			@@conf_handlers[level] = handler
		end
	end

	def self.set_min_log_level(level)
		@@conf_min_log_level = level
	end

	def self.debug(*args)
		make_error(:debug, *args)
	end

	def self.info(*args)
		make_error(:info, *args)
	end

	def self.warn(*args)
		make_error(:warn, *args)
	end

	def self.error(*args)
		make_error(:error, *args)
	end

	def self.fatal(*args)
		make_error(:fatal, *args)
	end

	private

	def self.make_error(level, *args)
		error = args[0].is_a?(CuvvaError) ? args[0] : CuvvaError.new(*args)
		handle_error(level, error)
		error
	end

	def self.handle_error(level, error)
		handler = @@conf_handlers[level]

		if handler
			handler.handle(level, error)
		else
			default_handler(level, error)
		end
	end

	def self.default_handler(level, error)
		return if ERROR_LEVELS[level] < ERROR_LEVELS[@@conf_min_log_level]

		error_string = "#{level}:#{error.to_json}"

		if [:warn, :error, :fatal].include?(level)
			STDERR.puts error_string
		else
			puts error_string
		end
	end
end
